import os
from urllib.parse import quote, quote_plus

from scrapgo import LinkRelayScraper
from scrapgo.utils.fileutils import cp
from .logger import logger


class BaseScraper(LinkRelayScraper):
    CACHE_NAME = 'CODINGFORENTREPRENEURS_CACHE'
    REQUEST_DELAY = 0
    RETRY_INTERVAL_SECONDS = 1, 2,

    def _save_media(self, url, save_to, overwrite=False):
        if overwrite is False:
            if os.path.exists(save_to):
                return
        try:
            r = self._get(url)
        except:
            logger.error(save_to)
        else:
            cp(r.content, save_to)



from .courses import CodingForCoursesScraper
from .projects import CodingForProjectsScraper


def download_projects(params):
    cf = CodingForProjectsScraper()
    cf.scrap(**params)


def download_courses(params):
    cf = CodingForCoursesScraper()
    cf.scrap(**params)



import os
from urllib.parse import quote, quote_plus

from scrapgo import LinkRelayScraper, url
from scrapgo.utils import urljoin, parse_src
from scrapgo.utils.fileutils import mkdir_p

from .utils import encodepath, transpath
from .base import BaseScraper
from .logger import logger


class CodingForCoursesScraper(BaseScraper):
    ROOT_URL = 'https://www.codingforentrepreneurs.com/api/web/courses'
    LINK_RELAY = [
        url(
            'https://www.codingforentrepreneurs.com/api/web/courses/',
            parser='topic_parser',
            name='topic'
        ),
        url(
            'https://www.codingforentrepreneurs.com/api/web/courses/',
            generator='section_url_renderer',
            parser='section_parser',
            name='section'
        ),
        url(
            'https://www.codingforentrepreneurs.com/api/web/projects/',
            generator='lecture_list_url_renderer',
            parser='lecture_list_parser',
            name='lecture_list'
        ),
        url(
            'https://www.codingforentrepreneurs.com/api/web/projects/',
            generator='lecture_url_renderer',
            parser='lecture_parser',
            name='lecture'
        )
    ]

    def topic_parser(self, response, **kwargs):
        json = response.json()

        def topic_json_parser(json):
            return [
                {
                    'topic_id': r['id'],
                    'topic_title': r['title'],
                    'topic_slug': r['slug'],
                    'topic_path': r['path']
                }
                for r in json
            ]
        topic_list = topic_json_parser(json)
        self.topic_list = topic_list
        return topic_list

    def section_url_renderer(self, parent_response, path, base_dir):
        for topic in self.topic_list:
            self.topic_slug = topic['topic_slug']
            path = f"{self.ROOT_URL}/{self.topic_slug}/"
            save_path = os.path.join(base_dir, 'courses', self.topic_slug)
            mkdir_p(save_path)
            yield path

    def section_parser(self, response, **kwargs):
        json = response.json()

        def section_json_parser(json):
            return [
                {
                    'section_id': r['id'],
                    'section_title': r['title'],
                    'section_slug': r['slug'],
                }
                for r in json['sections']
            ]
        section_list = section_json_parser(json)
        self.section_list = section_list
        return section_list

    def lecture_list_url_renderer(self, parent_response, path, base_dir):
        for section in self.section_list:
            self.section_slug = section['section_slug']
            path = f"{self.ROOT_URL}/{self.topic_slug}/{self.section_slug}/"
            save_path = os.path.join(
                base_dir, 'courses', self.topic_slug, self.section_slug
            )
            mkdir_p(save_path)
            yield path

    def lecture_list_parser(self, response, **kwargs):
        json = response.json()

        def lecture_list_json_parser(json):
            return [
                {
                    'lecture_id': r['id'],
                    'lecture_title': r['title'],
                    'lecture_slug': r['slug'],
                }
                for r in json['lessons']
            ]
        lecture_list = lecture_list_json_parser(json)
        self.lecture_list = lecture_list
        return lecture_list

    def lecture_url_renderer(self, parent_response, path, **kwargs):
        for lecture in self.lecture_list:
            self.lecture_slug = lecture['lecture_slug']
            path = f"{self.ROOT_URL}/{self.topic_slug}/{self.section_slug}/{self.lecture_slug}/"
            yield path

    def lecture_parser(self, response, base_dir):
        json = response.json()

        def lecture_json_parser(json):
            res = json
            return {
                'video_cdn': res['video_cdn'],
            }
        lecture = lecture_json_parser(json)
        url = lecture['video_cdn']
        fn = parse_src(url)
        save_to = os.path.join(
            base_dir, 'courses', self.topic_slug, self.section_slug, fn
        )
        save_to = transpath(save_to)
        url = encodepath(url)
        self._save_media(url, save_to)
        return lecture



from urllib.parse import urlparse, quote, urlunparse


def urlencode(url):
    map_encode_char = {
        '+': '%2B',
        '&': '%26'
    }
    for sep, repl in map_encode_char.items():
        url = url.replace(sep, repl)
    return url


def encodepath(url):
    map_encode_char = {
        '+': '%2B',
        '&': '%26'
    }
    p = urlparse(url)
    path = p.path
    u = p._replace(path=quote(path))
    return urlunparse(u)


def transpath(path):
    trantab = {
        '|': '[]'
    }
    for sep, repl in trantab.items():
        path = path.replace(sep, repl)
    return path



"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
import os
from codecs import open
from os import path

from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))


# def get_long_description():
#     with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
#         return f.read()


def package_files(directory):
    paths = []
    for (_path, _directories, _filenames) in os.walk(directory):
        for filename in _filenames:
            paths.append(os.path.join('..', _path, filename))
    return paths


setup(
    name='scrapgo',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.1.8',

    description='Scraping library by crawling link patterns, url and parsing',
    # long_description=get_long_description(),

    # The project's main homepage.
    # url='https://github.com/zwolf21/health2',

    # Author details
    author='HS Moon',
    author_email='mhs9089@gmail.com',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],

    # What does your project relate to?
    keywords='scrapgo',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['requests', 'requests-cache',
        'bs4', 'fake-useragent', 'jinja2', 'pandas', 'pymysql', 'sqlalchemy',
    ],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        'dev': [],
        'test': [],
    },

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={
        'scrapgo': package_files('scrapgo')
    },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    data_files=[],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.

)



import logging

logger = logging.getLogger('codingforentrepreneurs')

stream_hander = logging.StreamHandler()
logger.addHandler(stream_hander)
file_handler = logging.FileHandler('codingforentrepreneurs.log')
logger.addHandler(file_handler)